// DrawFormView.cpp : implementation file
//

#include "stdafx.h"
#include "backcolorformview.h"

#include "DrawDoc.h"

#include "DrawFormView.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawFormView

IMPLEMENT_DYNCREATE(CDrawFormView, CFormView)

CDrawFormView::CDrawFormView()
	: CFormView(CDrawFormView::IDD)
{
	//{{AFX_DATA_INIT(CDrawFormView)
	m_nX = 0;
	m_nY = 0;
	m_nShape = -1;
	//}}AFX_DATA_INIT

	// Set initial background color
	m_crBackColor = ::GetSysColor( COLOR_BTNFACE );
	m_brBack.CreateSolidBrush( m_crBackColor );
}

CDrawFormView::~CDrawFormView()
{
}

void CDrawFormView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDrawFormView)
	DDX_Text(pDX, IDC_X, m_nX);
	DDX_Text(pDX, IDC_Y, m_nY);
	DDX_CBIndex(pDX, IDC_SHAPE, m_nShape);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDrawFormView, CFormView)
	//{{AFX_MSG_MAP(CDrawFormView)
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	ON_COMMAND(ID_BACKGROUND_RED, OnBackgroundRed)
	ON_UPDATE_COMMAND_UI(ID_BACKGROUND_RED, OnUpdateBackgroundRed)
	ON_COMMAND(ID_BACKGROUND_GREEN, OnBackgroundGreen)
	ON_UPDATE_COMMAND_UI(ID_BACKGROUND_GREEN, OnUpdateBackgroundGreen)
	ON_COMMAND(ID_BACKGROUND_BLUE, OnBackgroundBlue)
	ON_UPDATE_COMMAND_UI(ID_BACKGROUND_BLUE, OnUpdateBackgroundBlue)
	ON_COMMAND(ID_BACKGROUND_GRAY, OnBackgroundGray)
	ON_UPDATE_COMMAND_UI(ID_BACKGROUND_GRAY, OnUpdateBackgroundGray)
	ON_WM_CTLCOLOR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrawFormView diagnostics

#ifdef _DEBUG
void CDrawFormView::AssertValid() const
{
	CFormView::AssertValid();
}

void CDrawFormView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CDrawDoc* CDrawFormView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDrawDoc)));
	return (CDrawDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawFormView message handlers

void CDrawFormView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	// 1 - Get the current data from our document
	CDrawDoc* pDoc = GetDocument();

	m_nX = pDoc->m_point.x;
	m_nY = pDoc->m_point.y;

	switch( pDoc->m_shape )
	{
	default :
		ASSERT( FALSE );	// This should never happen !
		break;

	case CDrawDoc::SQUARE :
		m_nShape = 0;
		break;

	case CDrawDoc::CIRCLE :
		m_nShape = 1;
		break;
	}

	// 2 - Show the current data in our controls
	UpdateData( FALSE );
}

void CDrawFormView::OnApply() 
{
	// 1 - Get the data from our controls
	UpdateData( TRUE );

	// 2 - Update the document state
	CDrawDoc* pDoc = GetDocument();

	pDoc->m_point.x = m_nX;
	pDoc->m_point.y = m_nY;

	switch( m_nShape )
	{
	default:
		ASSERT( FALSE );		// This should never happen !
		break;

	case 0:
		pDoc->m_shape = CDrawDoc::SQUARE;
		break;

	case 1:
		pDoc->m_shape = CDrawDoc::CIRCLE;
		break;
	}

	// 3 - Tell the document something happened
	pDoc->SetModifiedFlag( TRUE );

	// 4 - Synchronize all other views
	pDoc->UpdateAllViews( this );
}


void CDrawFormView::OnBackgroundRed() 
{
	// 1 - Change the background color to red
	m_crBackColor = RGB( 255, 0, 0 );
	m_brBack.DeleteObject();
	m_brBack.CreateSolidBrush( m_crBackColor );
	
	// 2 - Force the view to repaint
	Invalidate( TRUE );
}

void CDrawFormView::OnUpdateBackgroundRed(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio( m_crBackColor == RGB( 255, 0, 0 ) );
}

void CDrawFormView::OnBackgroundGreen() 
{
	// 1 - Change the background color to green
	m_crBackColor = RGB( 0, 255, 0 );
	m_brBack.DeleteObject();
	m_brBack.CreateSolidBrush( m_crBackColor );
	
	// 2 - Force the view to repaint
	Invalidate( TRUE );	
}

void CDrawFormView::OnUpdateBackgroundGreen(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio( m_crBackColor == RGB( 0, 255, 0 ) );	
}

void CDrawFormView::OnBackgroundBlue() 
{
	// 1 - Change the background color to blue
	m_crBackColor = RGB( 0, 0, 255 );
	m_brBack.DeleteObject();
	m_brBack.CreateSolidBrush( m_crBackColor );
	
	// 2 - Force the view to repaint
	Invalidate( TRUE );	
}

void CDrawFormView::OnUpdateBackgroundBlue(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio( m_crBackColor == RGB( 0, 0, 255 ) );	
}

void CDrawFormView::OnBackgroundGray() 
{
	m_crBackColor = ::GetSysColor( COLOR_BTNFACE );
	m_brBack.DeleteObject();
	m_brBack.CreateSolidBrush( m_crBackColor );
	Invalidate( TRUE );	
}

void CDrawFormView::OnUpdateBackgroundGray(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio( m_crBackColor == ::GetSysColor( COLOR_BTNFACE ) );	
}

HBRUSH CDrawFormView::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	switch( nCtlColor )
	{
	case CTLCOLOR_BTN:
	case CTLCOLOR_STATIC:
		pDC->SetBkMode( TRANSPARENT );
		// Fall through

	case CTLCOLOR_DLG:
		return (HBRUSH) m_brBack.GetSafeHandle();
	}

	return CFormView::OnCtlColor(pDC, pWnd, nCtlColor);
}
