// DrawView.cpp : implementation of the CDrawView class
//

#include "stdafx.h"
#include "DynamicCursor.h"

#include "DrawDoc.h"
#include "DrawView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawView

IMPLEMENT_DYNCREATE(CDrawView, CView)

BEGIN_MESSAGE_MAP(CDrawView, CView)
	//{{AFX_MSG_MAP(CDrawView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_DYNAMIC_CURSOR, OnDynamicCursor)
	ON_UPDATE_COMMAND_UI(ID_DYNAMIC_CURSOR, OnUpdateDynamicCursor)
	ON_WM_SETCURSOR()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrawView construction/destruction

CDrawView::CDrawView()
{
	m_bDynamicCursor = FALSE;

	m_hcurCross = AfxGetApp()->LoadStandardCursor( IDC_CROSS );
	ASSERT( m_hcurCross != NULL );

	m_hcurSquare = AfxGetApp()->LoadCursor( IDC_SQUARE );
	ASSERT( m_hcurSquare != NULL );

	m_hcurCircle = AfxGetApp()->LoadCursor( IDC_CIRCLE );
	ASSERT( m_hcurCircle != NULL );
}

CDrawView::~CDrawView()
{
}

BOOL CDrawView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDrawView drawing

void CDrawView::OnDraw(CDC* pDC)
{
	CDrawDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CBrush brush( RGB( 255, 0, 255 ) );
	CBrush* pOldBrush = pDC->SelectObject( &brush );

	CRect rect( pDoc->m_point.x-20, pDoc->m_point.y-20,
				pDoc->m_point.x+20, pDoc->m_point.y+20 );

	switch( pDoc->m_shape )
	{
	default :
		ASSERT( FALSE );  // We should never get here !
		break;

	case CDrawDoc::SQUARE :
		pDC->Rectangle( rect );
		break;

	case CDrawDoc::CIRCLE :
		pDC->Ellipse( rect );
		break;
	}

	CPen pen( PS_SOLID, 2, RGB( 0, 0, 0 ) );
	CPen* pOldPen = pDC->SelectObject( &pen );

	pDC->MoveTo( pDoc->m_point.x-3, pDoc->m_point.y-3 );
	pDC->LineTo( pDoc->m_point.x+3, pDoc->m_point.y+3 );

	pDC->MoveTo( pDoc->m_point.x-3, pDoc->m_point.y+3 );
	pDC->LineTo( pDoc->m_point.x+3, pDoc->m_point.y-3 );

	pDC->SelectObject( pOldPen );
	pDC->SelectObject( pOldBrush );
}

/////////////////////////////////////////////////////////////////////////////
// CDrawView printing

BOOL CDrawView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CDrawView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CDrawView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CDrawView diagnostics

#ifdef _DEBUG
void CDrawView::AssertValid() const
{
	CView::AssertValid();
}

void CDrawView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CDrawDoc* CDrawView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDrawDoc)));
	return (CDrawDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawView message handlers

void CDrawView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CDrawDoc* pDoc = GetDocument();
	pDoc->m_point = point;
	pDoc->UpdateAllViews( NULL );
	pDoc->SetModifiedFlag( TRUE );
	
	CView::OnLButtonDown(nFlags, point);
}


void CDrawView::OnDynamicCursor() 
{
	m_bDynamicCursor = !m_bDynamicCursor;
}

void CDrawView::OnUpdateDynamicCursor(CCmdUI* pCmdUI) 
{
	pCmdUI->SetRadio( m_bDynamicCursor );
}


BOOL CDrawView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	// 1 - Use dynamic cursor only in client area
	if( nHitTest == HTCLIENT )
	{
		// 2 - If no dynamic cursor, use cross as default
		if( !m_bDynamicCursor )
		{
			SetCursor( m_hcurCross );
			return TRUE;	// Cursor was set
		}
		else
		{
			// 3 - Select correct dynamic cursor
			switch( GetDocument()->m_shape )
			{
				default:
					ASSERT( FALSE );
					break;

				case CDrawDoc::SQUARE:
					SetCursor( m_hcurSquare );
					return TRUE;	// Cursor was set

				case CDrawDoc::CIRCLE:
					SetCursor( m_hcurCircle );
					return TRUE;	// Cursor was set
			}
		}
	}

	// 4 - Otherwise call base class to get standard cursor
	return CView::OnSetCursor(pWnd, nHitTest, message);
}
