// DrawFormView.cpp : implementation file
//

#include "stdafx.h"
#include "FormViewTooltips.h"

#include "DrawDoc.h"

#include "DrawFormView.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawFormView

IMPLEMENT_DYNCREATE(CDrawFormView, CFormView)

CDrawFormView::CDrawFormView()
	: CFormView(CDrawFormView::IDD)
{
	//{{AFX_DATA_INIT(CDrawFormView)
	m_nX = 0;
	m_nY = 0;
	m_nShape = -1;
	//}}AFX_DATA_INIT
}

CDrawFormView::~CDrawFormView()
{
}

void CDrawFormView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDrawFormView)
	DDX_Control(pDX, IDC_Y, m_edtY);
	DDX_Control(pDX, IDC_X, m_edtX);
	DDX_Control(pDX, IDC_SHAPE, m_cmbShape);
	DDX_Control(pDX, IDC_APPLY, m_cmdApply);
	DDX_Text(pDX, IDC_X, m_nX);
	DDX_Text(pDX, IDC_Y, m_nY);
	DDX_CBIndex(pDX, IDC_SHAPE, m_nShape);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDrawFormView, CFormView)
	//{{AFX_MSG_MAP(CDrawFormView)
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrawFormView diagnostics

#ifdef _DEBUG
void CDrawFormView::AssertValid() const
{
	CFormView::AssertValid();
}

void CDrawFormView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CDrawDoc* CDrawFormView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDrawDoc)));
	return (CDrawDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawFormView message handlers

void CDrawFormView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	// 1 - Get the current data from our document
	CDrawDoc* pDoc = GetDocument();

	m_nX = pDoc->m_point.x;
	m_nY = pDoc->m_point.y;

	switch( pDoc->m_shape )
	{
	default :
		ASSERT( FALSE );	// This should never happen !
		break;

	case CDrawDoc::SQUARE :
		m_nShape = 0;
		break;

	case CDrawDoc::CIRCLE :
		m_nShape = 1;
		break;
	}

	// 2 - Show the current data in our controls
	UpdateData( FALSE );
}

void CDrawFormView::OnApply() 
{
	// 1 - Get the data from our controls
	UpdateData( TRUE );

	// 2 - Update the document state
	CDrawDoc* pDoc = GetDocument();

	pDoc->m_point.x = m_nX;
	pDoc->m_point.y = m_nY;

	switch( m_nShape )
	{
	default:
		ASSERT( FALSE );		// This should never happen !
		break;

	case 0:
		pDoc->m_shape = CDrawDoc::SQUARE;
		break;

	case 1:
		pDoc->m_shape = CDrawDoc::CIRCLE;
		break;
	}

	// 3 - Tell the document something happened
	pDoc->SetModifiedFlag( TRUE );

	// 4 - Synchronize all other views
	pDoc->UpdateAllViews( this );
}


BOOL CDrawFormView::PreTranslateMessage(MSG* pMsg) 
{
	// Relay events to the tooltip control
	m_ToolTip.RelayEvent( pMsg );
	return CFormView::PreTranslateMessage(pMsg);
}

void CDrawFormView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	
	// 1 - Create the tooltip control
	if( !m_ToolTip.Create( this ) )
	{
		TRACE0( "Unable to create ToolTip control\n" );
		return;
	}

	// 2 - Add the form view controls to the tooltip
	m_ToolTip.AddTool( &m_edtX, IDS_TT_X );
	m_ToolTip.AddTool( &m_edtY, IDS_TT_Y );
	m_ToolTip.AddTool( &m_cmbShape, IDS_TT_SHAPE );
	m_ToolTip.AddTool( &m_cmdApply, IDS_TT_APPLY );

	// 3 - Activate the tooltip control
	m_ToolTip.Activate( TRUE );
}
