// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "CustomBars.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_VIEW_SHAPES_TOOLBAR, OnViewShapesToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHAPES_TOOLBAR, OnUpdateViewShapesToolbar)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

///////////////////////////////////////////////////////////
// EkIsBarVisible

BOOL EkIsBarVisible( CControlBar* pBar )
{
	ASSERT_VALID( pBar );

	return ( ( pBar->GetStyle() & WS_VISIBLE ) != 0 );
}

///////////////////////////////////////////////////////////
// EkDockBarNextTo

void EkDockBarNextTo(	CControlBar* pNewBar,
						CControlBar* pDockedBar,
						UINT nDockBarID = AFX_IDW_DOCKBAR_TOP )
{
	ASSERT_VALID( pDockedBar );
	ASSERT_VALID( pNewBar );

	// 1 - Find the frame where we will dock
	CFrameWnd* pFrame = pDockedBar->GetDockingFrame();

	// 2 - Force MFC to compute the positions
	// of the docked control bar(s)
	pFrame->RecalcLayout();

	// 3 - Compute rectangle of already "docked" bar
	CRect rect;
	pDockedBar->GetWindowRect( &rect );

	// 4 - Offset the rectangle slightly to the bottom right
	// so that the new bar will dock either to the right or
	// to the bottom of the existing bar (depending on the
	// side where this last bar is already docked)
	rect.OffsetRect(1,1);

	// 5 - Dock new bar to specified position
	pFrame->DockControlBar( pNewBar, nDockBarID, &rect );
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create standard toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndTBshapes.Create(this, WS_CHILD | WS_VISIBLE | CBRS_TOP, IDR_SHAPES) ||
		!m_wndTBshapes.LoadToolBar(IDR_SHAPES))
	{
		TRACE0("Failed to create shapes toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndTBdemo.Create(this, WS_CHILD | WS_VISIBLE | CBRS_TOP, IDR_DEMO_TOOLBAR) ||
		!m_wndTBdemo.LoadToolBar(IDR_DEMO_TOOLBAR))
	{
		TRACE0("Failed to create demo toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// Mainframe docking
	EnableDocking(CBRS_ALIGN_ANY);

	// Standard toolbar docking
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
					CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// Shapes toolbar docking
	m_wndTBshapes.SetBarStyle(m_wndTBshapes.GetBarStyle() |
					CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndTBshapes.EnableDocking(CBRS_ALIGN_ANY);
	EkDockBarNextTo( &m_wndTBshapes, &m_wndToolBar, AFX_IDW_DOCKBAR_TOP );
	m_wndTBshapes.SetWindowText( CString( LPCSTR( IDR_SHAPES ) ) );

	// Demo toolbar docking
	m_wndTBdemo.SetBarStyle(m_wndTBdemo.GetBarStyle() |
					CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndTBdemo.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndTBdemo, AFX_IDW_DOCKBAR_LEFT);

	// Create border bars
	if (	!m_wndBorderTop.Create( this, CBRS_TOP, IDW_BORDER_TOP, IDB_BORDER_TOP )
		||	!m_wndBorderBottom.Create( this, CBRS_BOTTOM, IDW_BORDER_BOTTOM, IDB_BORDER_BOTTOM )
		||	!m_wndBorderLeft.Create( this, CBRS_LEFT, IDW_BORDER_LEFT, IDB_BORDER_LEFT ) 
		||	!m_wndBorderRight.Create( this, CBRS_RIGHT, IDW_BORDER_RIGHT, IDB_BORDER_RIGHT ) )
	{
		TRACE0("Unable to create border bars\n");
		return -1;      // fail to create
	}

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CMDIFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnViewShapesToolbar() 
{
	ShowControlBar( &m_wndTBshapes, !EkIsBarVisible( &m_wndTBshapes ), FALSE );
}

void CMainFrame::OnUpdateViewShapesToolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( EkIsBarVisible( &m_wndTBshapes ) );	
}
