// DrawDoc.cpp : implementation of the CDrawDoc class
//

#include "stdafx.h"
#include "DrawInStatusBarPane.h"

#include "DrawDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc

IMPLEMENT_DYNCREATE(CDrawDoc, CDocument)

BEGIN_MESSAGE_MAP(CDrawDoc, CDocument)
	//{{AFX_MSG_MAP(CDrawDoc)
	ON_COMMAND(ID_DRAW_SQUARE, OnDrawSquare)
	ON_COMMAND(ID_DRAW_CIRCLE, OnDrawCircle)
	ON_UPDATE_COMMAND_UI(ID_DRAW_SQUARE, OnUpdateDrawSquare)
	ON_UPDATE_COMMAND_UI(ID_DRAW_CIRCLE, OnUpdateDrawCircle)

	// Update status bar indicators
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_SHAPE, OnUpdateIndicatorShape)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_SHAPE_BITMAP, OnUpdateIndicatorShapeBitmap)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_SHAPE_DRAW, OnUpdateIndicatorShapeDraw)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc construction/destruction

CDrawDoc::CDrawDoc()
{
	m_point = CPoint( 30, 30 );
	m_shape = SQUARE;
}

CDrawDoc::~CDrawDoc()
{
}

BOOL CDrawDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc serialization

void CDrawDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << m_point << (DWORD) m_shape;
	}
	else
	{
		ar >> m_point >> (DWORD&) m_shape;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc diagnostics

#ifdef _DEBUG
void CDrawDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDrawDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc commands

void CDrawDoc::OnDrawSquare() 
{
	m_shape = SQUARE;
	UpdateAllViews( NULL );
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnDrawCircle() 
{
	m_shape = CIRCLE;
	UpdateAllViews( NULL );
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnUpdateDrawSquare(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_shape == SQUARE );
}

void CDrawDoc::OnUpdateDrawCircle(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_shape == CIRCLE );
}

void CDrawDoc::OnUpdateIndicatorShape(CCmdUI* pCmdUI)
{
	pCmdUI->SetText( ( m_shape == SQUARE ) ? _T( "SQUARE" ) :  _T( "CIRCLE" ) );

	pCmdUI->Enable( TRUE );
}

///////////////////////////////////////////////////////////
// EkDrawBitmap

void EkDrawBitmap( CDC* pDC, CBitmap* pBitmap, CRect rect, BOOL bCenter = TRUE )
{
	ASSERT_VALID( pDC );
	ASSERT_VALID( pBitmap );

	// 1 - Create compatible memory DC
   CDC dcMem;
   dcMem.CreateCompatibleDC( pDC );

   // 2 - Select bitmap into memory DC
   CBitmap* pOldBitmap = dcMem.SelectObject( pBitmap );

   if( bCenter )
   {
	   // 3 - Center bitmap in destination rectangle
	   BITMAP bitmap;
	   pBitmap->GetObject( sizeof( BITMAP ), &bitmap);
	   CSize sizeBitmap( bitmap.bmWidth, bitmap.bmHeight );
	   CSize diff = rect.Size() - sizeBitmap;
	   rect.DeflateRect( diff.cx / 2, diff.cy / 2 );
   }

   // 4 - Blit the bitmap from the memory DC to the real DC
   pDC->BitBlt( rect.left, rect.top,
             rect.Width(), rect.Height(),
             &dcMem, 0, 0, SRCCOPY );

   // 5 - Deselect bitmap from memory DC
   dcMem.SelectObject( pOldBitmap );
}

void CDrawDoc::OnUpdateIndicatorShapeBitmap(CCmdUI* pCmdUI)
{
	// 1 - Prevent status bar from drawing text
	pCmdUI->SetText( _T( "" ) );
    pCmdUI->Enable( TRUE );    

	// 2 - Retrieve pointer to status bar object
    CStatusBar* psb = static_cast<CStatusBar*>( pCmdUI->m_pOther );
	ASSERT_KINDOF( CStatusBar, psb );
	ASSERT_VALID( psb );
                                                
	// 3 - Compute drawing zone rectangle
    CRect rect;
    psb->GetItemRect( pCmdUI->m_nIndex, &rect );
	rect.DeflateRect( 2, 2 );

	// 4 - Draw current shape in pane
    CClientDC dc( psb );

	CBitmap bmp;
	bmp.LoadBitmap( m_shape == SQUARE ? IDB_SQUARE : IDB_CIRCLE );

	EkDrawBitmap( &dc, &bmp, rect );
}

void CDrawDoc::OnUpdateIndicatorShapeDraw(CCmdUI* pCmdUI)
{
	// 1 - Prevent status bar from drawing text
	pCmdUI->SetText( _T( "" ) );
    pCmdUI->Enable( TRUE );    

	// 2 - Retrieve pointer to status bar object
    CStatusBar* psb = static_cast<CStatusBar*>( pCmdUI->m_pOther );
	ASSERT_KINDOF( CStatusBar, psb );
	ASSERT_VALID( psb );
                                                
	// 3 - Compute drawing zone rectangle
    CRect rect;
    psb->GetItemRect( pCmdUI->m_nIndex, &rect );
	rect.DeflateRect( 2, 2 );

	// 4 - Draw current shape in pane
    CClientDC dc( psb );

	// Erase destination rectangle
	CBrush brBack( ::GetSysColor( COLOR_3DFACE ) );
	CBrush* pOldBrush = dc.SelectObject( &brBack );

	dc.PatBlt( rect.left, rect.top, rect.Width(), rect.Height(), PATCOPY );

	// Draw shape
	CBrush brShape( RGB( 255, 0, 255 ) );
	dc.SelectObject( &brShape );

	switch( m_shape )
	{
	default :
		ASSERT( FALSE );  // We should never get here !
		break;

	case SQUARE :
		dc.Rectangle( &rect );
		break;

	case CIRCLE :
		dc.Ellipse( &rect );
		break;
	}

	dc.SelectObject( pOldBrush );
}
