// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "MaximizeMessagePane.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_VIEW_SHAPES_TOOLBAR, OnViewShapesToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHAPES_TOOLBAR, OnUpdateViewShapesToolbar)
	ON_WM_INITMENUPOPUP()
	ON_WM_MENUSELECT()
	ON_COMMAND(ID_DEMO_MAXIMIZE_MESSAGE_PANE, OnDemoMaximizeMessagePane)
	ON_UPDATE_COMMAND_UI(ID_DEMO_MAXIMIZE_MESSAGE_PANE, OnUpdateDemoMaximizeMessagePane)
	ON_UPDATE_COMMAND_UI(ID_DEMO_LONG_PROMPT, OnUpdateDemoLongPrompt)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator

	ID_INDICATOR_X,
	ID_INDICATOR_Y,
	ID_INDICATOR_SHAPE,

	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

///////////////////////////////////////////////////////////
// EkIsBarVisible

BOOL EkIsBarVisible( CControlBar* pBar )
{
	ASSERT_VALID( pBar );

	return ( ( pBar->GetStyle() & WS_VISIBLE ) != 0 );
}

///////////////////////////////////////////////////////////
// EkDockBarNextTo

void EkDockBarNextTo(	CControlBar* pNewBar,
						CControlBar* pDockedBar,
						UINT nDockBarID = AFX_IDW_DOCKBAR_TOP )
{
	ASSERT_VALID( pDockedBar );
	ASSERT_VALID( pNewBar );

	// 1 - Find the frame where we will dock
	CFrameWnd* pFrame = pDockedBar->GetDockingFrame();

	// 2 - Force MFC to compute the positions
	// of the docked control bar(s)
	pFrame->RecalcLayout();

	// 3 - Compute rectangle of already "docked" bar
	CRect rect;
	pDockedBar->GetWindowRect( &rect );

	// 4 - Offset the rectangle slightly to the bottom right
	// so that the new bar will dock either to the right or
	// to the bottom of the existing bar (depending on the
	// side where this last bar is already docked)
	rect.OffsetRect(1,1);

	// 5 - Dock new bar to specified position
	pFrame->DockControlBar( pNewBar, nDockBarID, &rect );
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_cxMessagePaneWidth = -1;
	m_bInMenuSelect = FALSE;

	m_bMaximizeMessagePane = TRUE;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create standard toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndTBshapes.Create(this, WS_CHILD | WS_VISIBLE | CBRS_TOP, IDR_SHAPES) ||
		!m_wndTBshapes.LoadToolBar(IDR_SHAPES))
	{
		TRACE0("Failed to create shapes toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndTBdemo.Create(this, WS_CHILD | WS_VISIBLE | CBRS_TOP, IDR_DEMO_TOOLBAR) ||
		!m_wndTBdemo.LoadToolBar(IDR_DEMO_TOOLBAR))
	{
		TRACE0("Failed to create demo toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// Mainframe docking
	EnableDocking(CBRS_ALIGN_ANY);

	// Standard toolbar docking
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
					CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	// Shapes toolbar docking
	m_wndTBshapes.SetBarStyle(m_wndTBshapes.GetBarStyle() |
					CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndTBshapes.EnableDocking(CBRS_ALIGN_ANY);
	EkDockBarNextTo( &m_wndTBshapes, &m_wndToolBar, AFX_IDW_DOCKBAR_TOP );
	m_wndTBshapes.SetWindowText( CString( LPCSTR( IDR_SHAPES ) ) );

	// Demo toolbar docking
	m_wndTBdemo.SetBarStyle(m_wndTBdemo.GetBarStyle() |
					CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	m_wndTBdemo.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndTBdemo, AFX_IDW_DOCKBAR_LEFT);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CMDIFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnViewShapesToolbar() 
{
	ShowControlBar( &m_wndTBshapes, !EkIsBarVisible( &m_wndTBshapes ), FALSE );
}

void CMainFrame::OnUpdateViewShapesToolbar(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( EkIsBarVisible( &m_wndTBshapes ) );	
}

void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
{
  	CMDIFrameWnd::OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);

	if( !m_bMaximizeMessagePane )
		return;
  
	// 1 - Check if in "menu selection" mode and message
	// pane not stretched
	if( m_cxMessagePaneWidth == -1 && m_bInMenuSelect )
	{
		// 2 - Retrieve "normal" message pane style and width
		m_wndStatusBar.GetPaneInfo(0, m_nMessagePaneID, 
			m_nMessagePaneStyle, m_cxMessagePaneWidth);

		// 3 - Stretch message pane width to cover entire status bar
		CRect rc;
		m_wndStatusBar.GetWindowRect( &rc );
		m_wndStatusBar.SetPaneInfo(0, m_nMessagePaneID, 
			SBPS_NOBORDERS|SBPS_STRETCH, rc.Width() );
	}
}

void CMainFrame::OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu)
{
  	CMDIFrameWnd::OnMenuSelect(nItemID, nFlags, hSysMenu);
  
	if( !m_bMaximizeMessagePane )
		return;

	// 1 - Was the menu dismissed?
	BOOL bMenuCanceled = ( nFlags == 0xFFFF && hSysMenu == 0 );
	if( bMenuCanceled && m_cxMessagePaneWidth != -1 )
	{
		// 2a - Restore message pane style and width
		m_bInMenuSelect = FALSE;
		m_wndStatusBar.SetPaneInfo(0, 
			m_nMessagePaneID, m_nMessagePaneStyle, m_cxMessagePaneWidth);
		m_cxMessagePaneWidth = -1;   // Guard (illegal) value
	}
	else 
	{
		// 2b - Go into "menu selection" mode
		m_bInMenuSelect = TRUE;
	}
}


void CMainFrame::OnDemoMaximizeMessagePane() 
{
	m_bMaximizeMessagePane = !m_bMaximizeMessagePane;
}

void CMainFrame::OnUpdateDemoMaximizeMessagePane(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_bMaximizeMessagePane );
}

void CMainFrame::OnUpdateDemoLongPrompt(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( TRUE );	
}
