// DrawDoc.cpp : implementation of the CDrawDoc class
//

#include "stdafx.h"
#include "ContextMenu.h"

#include "DrawDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc

IMPLEMENT_DYNCREATE(CDrawDoc, CDocument)

BEGIN_MESSAGE_MAP(CDrawDoc, CDocument)
	//{{AFX_MSG_MAP(CDrawDoc)
	ON_COMMAND(ID_DRAW_SQUARE, OnDrawSquare)
	ON_COMMAND(ID_DRAW_CIRCLE, OnDrawCircle)
	ON_UPDATE_COMMAND_UI(ID_DRAW_SQUARE, OnUpdateDrawSquare)
	ON_UPDATE_COMMAND_UI(ID_DRAW_CIRCLE, OnUpdateDrawCircle)
	//}}AFX_MSG_MAP

	// Color handling
	ON_COMMAND_RANGE(ID_COLOR0, ID_COLOR15, OnColor)
	ON_UPDATE_COMMAND_UI_RANGE(ID_COLOR0, ID_COLOR15, OnUpdateColor)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc construction/destruction

CDrawDoc::CDrawDoc()
{
	m_point = CPoint( 30, 30 );
	m_shape = SQUARE;
	m_crColor = GetColorRef( ID_COLOR13 );	// fuchsia
}

CDrawDoc::~CDrawDoc()
{
}

BOOL CDrawDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc serialization

void CDrawDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << m_point << (DWORD) m_shape << (DWORD) m_crColor;
	}
	else
	{
		ar >> m_point >> (DWORD&) m_shape >> (DWORD&) m_crColor;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc diagnostics

#ifdef _DEBUG
void CDrawDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDrawDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc commands

void CDrawDoc::OnDrawSquare() 
{
	m_shape = SQUARE;
	UpdateAllViews( NULL );
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnDrawCircle() 
{
	m_shape = CIRCLE;
	UpdateAllViews( NULL );
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnUpdateDrawSquare(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_shape == SQUARE );
}

void CDrawDoc::OnUpdateDrawCircle(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_shape == CIRCLE );
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc color handling

int CDrawDoc::s_ColorMap[] =
					{
						0, 		//black
						1, 		//dark red
						2, 		//dark green
						3, 		//light brown
						4, 		//dark blue
						5, 		//purple
						6, 		//dark cyan
						12, 	//gray
						7, 		//light gray
						13, 	//red
						14, 	//green
						15, 	//yellow
						16, 	//blue
						17, 	//magenta
						18, 	//cyan
						19	 	//white
					};

COLORREF CDrawDoc::GetColorRef( UINT nID )
{
	ASSERT( nID >= ID_COLOR0 );
	ASSERT( nID <= ID_COLOR15 );
	CPalette* pPal = CPalette::FromHandle(
							(HPALETTE) GetStockObject( DEFAULT_PALETTE ) );
	ASSERT( pPal != NULL );
	PALETTEENTRY pe;
	if( pPal->GetPaletteEntries( s_ColorMap[ nID-ID_COLOR0 ], 1, &pe ) != 0 )
	{
		return RGB( pe.peRed, pe.peGreen, pe.peBlue );
	}
	else
	{
		TRACE1( "Unable to find palette color entry for ID=%d\n", nID );
		return ::GetSysColor( COLOR_WINDOWTEXT );
	}
}

void CDrawDoc::OnColor( UINT nID )
{
	m_crColor = GetColorRef( nID );
	UpdateAllViews( NULL );
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnUpdateColor(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( GetColorRef( pCmdUI->m_nID ) == m_crColor );
}

