// DrawDoc.cpp : implementation of the CDrawDoc class
//

#include "stdafx.h"
#include "DynamicPagination.h"

#include "DrawDoc.h"

#include "ColorMenu.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc

IMPLEMENT_DYNCREATE(CDrawDoc, CDocument)

BEGIN_MESSAGE_MAP(CDrawDoc, CDocument)
	//{{AFX_MSG_MAP(CDrawDoc)
	ON_COMMAND(ID_DRAW_SQUARE, OnDrawSquare)
	ON_COMMAND(ID_DRAW_CIRCLE, OnDrawCircle)
	ON_UPDATE_COMMAND_UI(ID_DRAW_SQUARE, OnUpdateDrawSquare)
	ON_UPDATE_COMMAND_UI(ID_DRAW_CIRCLE, OnUpdateDrawCircle)
	//}}AFX_MSG_MAP

	// Color handling
	ON_COMMAND_RANGE(ID_COLOR0, ID_COLOR15, OnColor)
	ON_UPDATE_COMMAND_UI_RANGE(ID_COLOR0, ID_COLOR15, OnUpdateColor)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc construction/destruction

CDrawDoc::CDrawDoc()
{
	m_shape = CShape::SQUARE;
	m_crColor = CColorMenu::GetColorRef( ID_COLOR13 );	// fuchsia
}

CDrawDoc::~CDrawDoc()
{
}

BOOL CDrawDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

void CDrawDoc::DeleteContents() 
{
	for( int i = 0; i < m_Shapes.GetSize(); ++i )
	{
		delete m_Shapes.GetAt( i );
	}

	m_Shapes.RemoveAll();
	
	CDocument::DeleteContents();
}

void CDrawDoc::AddShape( CPoint point )
{
	CShape* pShape = new CShape( point, m_crColor, m_shape );
	m_Shapes.Add( pShape );

	SetModifiedFlag( TRUE );
	UpdateAllViews( NULL );
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc serialization

void CDrawDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << (DWORD) m_shape << (DWORD) m_crColor;
	}
	else
	{
		ar >> (DWORD&) m_shape >> (DWORD&) m_crColor;
	}

	m_Shapes.Serialize( ar );
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc diagnostics

#ifdef _DEBUG
void CDrawDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDrawDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc commands

void CDrawDoc::OnDrawSquare() 
{
	m_shape = CShape::SQUARE;
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnDrawCircle() 
{
	m_shape = CShape::CIRCLE;
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnUpdateDrawSquare(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_shape == CShape::SQUARE );
}

void CDrawDoc::OnUpdateDrawCircle(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( m_shape == CShape::CIRCLE );
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc color handling

void CDrawDoc::OnColor( UINT nID )
{
	m_crColor = CColorMenu::GetColorRef( nID );
	SetModifiedFlag( TRUE );
}

void CDrawDoc::OnUpdateColor(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck( CColorMenu::GetColorRef( pCmdUI->m_nID ) == m_crColor );
}

/////////////////////////////////////////////////////////////////////////////
// Simulate on-the-fly "queries"

void CDrawDoc::InitQuery()
{
	m_nCurElement = 0;
}

const CShape* CDrawDoc::GetNextElement()
{
	ASSERT( m_nCurElement < m_Shapes.GetSize() );
	return m_Shapes.GetAt( m_nCurElement++ );
}

BOOL CDrawDoc::IsQueryEnded()
{
	return ( m_nCurElement >= m_Shapes.GetSize() );
}

