#include "stdafx.h"
#include "PrintMono.h"

#include "Shape.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CShape

IMPLEMENT_SERIAL(CShape, CObject, 1)

CShape::CShape( CPoint ptCenter, COLORREF crColor, SHAPE shape )
{
	m_ptCenter = ptCenter;
	m_crColor = crColor;
	m_shape = shape;
}


CShape::CShape()
{
	m_ptCenter = CPoint( 0, 0 );
	m_crColor = 0;
	m_shape = SQUARE;
}


void CShape::Draw( CDC* pDC ) const
{
	DrawAt( pDC, m_ptCenter.x, m_ptCenter.y );
}


BOOL EkIsMonoPreview( CDC* pDC )
{
	ASSERT_VALID( pDC );

	// Check for monochrome device context
	BOOL bMono = ( pDC->GetDeviceCaps( NUMCOLORS ) == 2 );

	// Check for print preview mode
	BOOL bPreview = ( pDC->m_hDC != pDC->m_hAttribDC );

	return( bMono && bPreview );
}


COLORREF EkColorRefToGray( COLORREF crColor )
{
	// Extract (r, g, b) factors from COLORREF
	BYTE red = GetRValue( crColor );
	BYTE green = GetGValue( crColor );
	BYTE blue = GetBValue( crColor );

	// Convert (r, g, b) to gray using standard formula
	BYTE gray = static_cast< BYTE >( ( red*30 + green*59 + blue*11 ) / 100 );
	return RGB( gray, gray, gray );
}


void CShape::DrawAt( CDC* pDC, int x, int y ) const
{
	ASSERT_VALID( pDC );

	// Select optional gray fill color for print preview
	// on monochrome printer
	COLORREF crFill = EkIsMonoPreview( pDC ) ?
							EkColorRefToGray( m_crColor ) :
							m_crColor;

	CBrush brFill( crFill );
	CBrush* pOldBrush = pDC->SelectObject( &brFill );

	CRect rect( x-HALF_SIZE, y-HALF_SIZE, x+HALF_SIZE, y+HALF_SIZE );

	switch( m_shape )
	{
	default:
		ASSERT( FALSE );	// unknown shape type
		break;

	case SQUARE:
		pDC->Rectangle( rect );
		break;

	case CIRCLE:
		pDC->Ellipse( rect );
		break;
	}

	CPen pen( PS_SOLID, 6, RGB( 0, 0, 0 ) );
	CPen* pOldPen = pDC->SelectObject( &pen );

	pDC->MoveTo( x-10, y-10 );
	pDC->LineTo( x+10, y+10 );

	pDC->MoveTo( x-10, y+10 );
	pDC->LineTo( x+10, y-10 );

	pDC->SelectObject( pOldPen );
	pDC->SelectObject( pOldBrush );
}


void CShape::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		ar << m_ptCenter << (DWORD) m_crColor << (DWORD) m_shape;
	}
	else
	{
		ar >> m_ptCenter >> (DWORD&) m_crColor >> (DWORD&) m_shape;
	}
}


CString CShape::GetShapeName() const
{
	CString strShape;
	switch( m_shape )
	{
	default:
		ASSERT( FALSE );	// unknown shape type
		break;

	case SQUARE:
		strShape = _T( "Square" );
		break;

	case CIRCLE:
		strShape = _T( "Circle" );
		break;
	}

	return strShape;
}

